@@ -1,20 +1,19 @@
 from .core.buffer import PysFileBuffer
-from .core.cache import undefined
+from .core.cache import path, undefined
 from .core.constants import DEFAULT, DEBUG, NO_COLOR
 from .core.handlers import handle_execute
-from .core.highlight import HLFMT_HTML, HLFMT_ANSI, pys_highlight
+from .core.highlight import HLFMT_HTML, HLFMT_ANSI, HLFMT_BBCODE, pys_highlight
 from .core.runner import pys_runner, pys_shell
-from .core.symtab import build_symbol_table
-from .core.utils.general import normalize_path
+from .core.symtab import new_symbol_table
+from .core.utils.module import get_module_name_from_path, set_python_path
+from .core.utils.path import getcwd, normpath
 from .core.version import __version__
 
 from argparse import ArgumentParser
-
-from sys import executable, version_info, exit, setrecursionlimit
-from os.path import basename, splitext
+from sys import executable, stderr, version_info, exit, setrecursionlimit
 
 parser = ArgumentParser(
-    prog=splitext(basename(executable))[0] + ' -m pyscript',
+    prog=get_module_name_from_path(executable) + ' -m pyscript',
     description='PyScript Launcher for Python Version ' + '.'.join(map(str, version_info))
 )
 
@@ -53,7 +52,7 @@ parser.add_argument(
 
 parser.add_argument(
     '-l', '--highlight',
-    choices=('html', 'ansi'),
+    choices=('html', 'ansi', 'bbcode'),
     default=None,
     help='generate PyScript highlight code from a file'
 )
@@ -71,16 +70,20 @@ parser.add_argument(
     help="set a python recursion limit"
 )
 
+def argument_error(argument, message):
+    parser.print_usage(stderr)
+    parser.exit(2, f"{parser.prog}: error: argument {argument}: {message}\n")
+
 args = parser.parse_args()
 
 if args.highlight and args.file is None:
-    parser.error("-l, --highlight: file path require")
+    argument_error("-l/--highlight", "file path require")
 
 if args.py_recursion is not None:
     try:
         setrecursionlimit(args.py_recursion)
     except BaseException as e:
-        parser.error(f"-r, --py-recursion: {e}")
+        argument_error("-r/--py-recursion", e)
 
 code = 0
 flags = DEFAULT
@@ -90,83 +93,66 @@ if args.debug:
 if args.no_color:
     flags |= NO_COLOR
 
+set_python_path(getcwd())
+
 if args.file is not None:
-    path = normalize_path(args.file)
+    path = normpath(args.file)
 
     try:
         with open(path, 'r', encoding='utf-8') as file:
             file = PysFileBuffer(file, path)
-
     except FileNotFoundError:
         parser.error(f"can't open file {path!r}: No such file or directory")
-
     except PermissionError:
         parser.error(f"can't open file {path!r}: Permission denied.")
-
     except IsADirectoryError:
         parser.error(f"can't open file {path!r}: Path is not a file.")
-
     except NotADirectoryError:
         parser.error(f"can't open file {path!r}: Attempting to access directory from file.")
-
     except (OSError, IOError):
         parser.error(f"can't open file {path!r}: Attempting to access a system directory or file.")
-
     except UnicodeDecodeError:
         parser.error(f"can't read file {path!r}: Bad file.")
-
     except BaseException as e:
         parser.error(f"file {path!r}: Unexpected error: {e}")
 
     if args.highlight:
+        format_map = {
+            'html': HLFMT_HTML,
+            'ansi': HLFMT_ANSI,
+            'bbcode': HLFMT_BBCODE
+        }
+
         try:
-            print(
-                pys_highlight(
-                    file,
-                    HLFMT_HTML if args.highlight == 'html' else HLFMT_ANSI
-                )
-            )
+            print(pys_highlight(file, format_map[args.highlight]))
         except BaseException as e:
             parser.error(f"file {path!r}: Tokenize error: {e}")
 
     else:
-        symtab = build_symbol_table(file)
-        symtab.set('__name__', '__main__')
-
         result = pys_runner(
             file=file,
             mode='exec',
-            symbol_table=symtab,
+            symbol_table=new_symbol_table(file=file.name, name='__main__')[0],
             flags=flags
         )
 
-        code = handle_execute(result)
-
         if args.inspect:
-            code = pys_shell(
-                globals=result.context.symbol_table,
-                flags=result.context.flags
-            )
+            code = pys_shell(result.context.symbol_table, result.context.flags)
+        else:
+            code = handle_execute(result)
 
 elif args.command is not None:
     file = PysFileBuffer(args.command)
-
-    symtab = build_symbol_table(file)
-    symtab.set('__name__', '__main__')
-
     code = handle_execute(
         pys_runner(
             file=file,
             mode='exec',
-            symbol_table=symtab,
+            symbol_table=new_symbol_table(file=file.name, name='__main__')[0],
             flags=flags
         )
     )
 
 else:
-    code = pys_shell(
-        globals=undefined,
-        flags=flags
-    )
+    code = pys_shell(undefined, flags)
 
 exit(code)
\ No newline at end of file
