@@ -1,33 +1,24 @@
 from .buffer import PysFileBuffer
-from .cache import loading_modules, modules, library, undefined
-from .constants import LIBRARY_PATH
-from .exceptions import PysShouldReturn
+from .cache import loading_modules, modules, path, undefined
+from .checks import is_blacklist_python_builtins, is_private_attribute
+from .exceptions import PysSignal
 from .handlers import handle_call
-from .objects import PysModule, PysPythonFunction
+from .objects import PysPythonFunction
 from .results import PysRunTimeResult
-from .symtab import build_symbol_table
-from .utils.constants import BLACKLIST_PYTHON_BUILTINS
-from .utils.general import (
-    tostr,
-    normalize_path,
-    is_object_of as isobjectof
-)
+from .symtab import new_symbol_table
+from .utils.generic import is_object_of as isobjectof
+from .utils.module import get_module_name_from_path, get_module_path, set_python_path
+from .utils.path import getcwd, normpath
+from .utils.string import normstr
 
 from math import isclose
-from importlib import import_module as pyimport
-from os import getcwd
-from os.path import (
-    dirname as pdirname,
-    join as pjoin,
-    isdir as pisdir,
-    exists as pexists,
-    basename as pbasename
-)
+from importlib import import_module
+from os.path import dirname
+from types import ModuleType
 
 import builtins
 
 def _supported_method(pyfunc, object, name, *args, **kwargs):
-
     method = getattr(object, name, undefined)
     if method is undefined:
         return False, None
@@ -35,14 +26,12 @@ def _supported_method(pyfunc, object, name, *args, **kwargs):
     if callable(method):
         code = pyfunc.__code__
         handle_call(method, code.context, code.position)
-
         try:
             result = method(*args, **kwargs)
-            if result is NotImplemented:
-                return False, None
-            return True, result
+            if result is not NotImplemented:
+                return True, result
         except NotImplementedError:
-            return False, None
+            pass
 
     return False, None
 
@@ -90,82 +79,89 @@ help = _Helper()
 
 @PysPythonFunction
 def require(pyfunc, name):
-    name = tostr(name)
+    name, *other_components = normstr(name).split('>')
+    external = True
 
-    if name == '_pyscript':
-        from .. import core
-        return core
-
-    elif name == 'builtins':
-        return pys_builtins
-
-    normalize = True
-
-    if name in library:
-        path = pjoin(LIBRARY_PATH, name)
-        if not pisdir(path):
-            path += '.pys'
-        if pexists(path):
-            normalize = False
-
-    if normalize:
-        path = normalize_path(
-            pdirname(pyfunc.__code__.context.file.name) or getcwd(),
-            name,
-            absolute=False
+    for p in path:
+        path_package = get_module_path(normpath(p, name, absolute=False))
+        if path_package is not None:
+            break
+    else:
+        path_package = get_module_path(
+            normpath(
+                dirname(pyfunc.__code__.context.file.name) or getcwd(),
+                name,
+                absolute=False
+            )
         )
 
-    module_name = pbasename(path)
+    if path_package is None:
 
-    if pisdir(path):
-        path = pjoin(path, '__init__.pys')
+        if name == '_pyscript':
+            from .. import core as module
+            external = False
 
-    if path in loading_modules:
-        raise ImportError(
-            f"cannot import module name {module_name!r} "
-            f"from partially initialized module {pyfunc.__code__.context.file.name!r}, mostly during circular import"
-        )
+        elif name == 'builtins':
+            module = pys_builtins
+            external = False
 
-    loading_modules.add(path)
+        else:
+            path_package = name
 
-    try:
+    if external:
 
-        package = modules.get(path, None)
-
-        if package is None:
-            try:
-                with open(path, 'r', encoding='utf-8') as file:
-                    file = PysFileBuffer(file.read(), path)
-            except FileNotFoundError:
-                raise ModuleNotFoundError(f"No module named {module_name!r}")
-            except BaseException as e:
-                raise ImportError(f"Cannot import module named {module_name!r}: {e}")
-
-            symtab = build_symbol_table(file)
-
-            package = PysModule('')
-            package.__dict__ = symtab.symbols
-
-            from .runner import pys_runner
-
-            result = pys_runner(
-                file=file,
-                mode='exec',
-                symbol_table=symtab,
-                context_parent=pyfunc.__code__.context,
-                context_parent_entry_position=pyfunc.__code__.position
+        if path_package in loading_modules:
+            raise ImportError(
+                f"cannot import module name {name!r} "
+                f"from partially initialized module {pyfunc.__code__.context.file.name!r}, "
+                "mostly during circular import"
             )
 
-            if result.error:
-                raise PysShouldReturn(PysRunTimeResult().failure(result.error))
+        module = modules.get(path_package, None)
 
-            modules[path] = package
+        if module is None:
 
-        return package
+            try:
+                loading_modules.add(path_package)
 
-    finally:
-        if path in loading_modules:
-            loading_modules.remove(path)
+                try:
+                    with open(path_package, 'r', encoding='utf-8') as file:
+                        file = PysFileBuffer(file, path_package)
+                except FileNotFoundError as e:
+                    raise ModuleNotFoundError(f"No module named {name!r}") from e
+                except BaseException as e:
+                    raise ImportError(f"Cannot import module named {name!r}: {e}") from e
+
+                symtab, module = new_symbol_table(file=file.name, name=get_module_name_from_path(name))
+                code = pyfunc.__code__
+
+                from .runner import pys_runner
+
+                result = pys_runner(
+                    file=file,
+                    mode='exec',
+                    symbol_table=symtab,
+                    context_parent=code.context,
+                    context_parent_entry_position=code.position
+                )
+
+                if result.error:
+                    raise PysSignal(PysRunTimeResult().failure(result.error))
+
+                modules[path_package] = module
+            finally:
+                if path_package in loading_modules:
+                    loading_modules.remove(path_package)
+
+    for component in other_components:
+        module = getattr(module, component)
+
+    return module
+
+@PysPythonFunction
+def pyimport(pyfunc, name):
+    set_python_path(dirname(pyfunc.__code__.context.file.name))
+    return import_module(name)
 
 @PysPythonFunction
 def globals(pyfunc):
@@ -180,26 +176,31 @@ def globals(pyfunc):
 
         return result
 
-    else:
-        return pyfunc.__code__.context.symbol_table.symbols
+    return pyfunc.__code__.context.symbol_table.symbols
 
 @PysPythonFunction
 def locals(pyfunc):
     return pyfunc.__code__.context.symbol_table.symbols
 
+pyvars = builtins.vars
+
 @PysPythonFunction
 def vars(pyfunc, object=None):
-    if object is None:
-        return pyfunc.__code__.context.symbol_table.symbols
+    return (
+        pyfunc.__code__.context.symbol_table.symbols
+        if object is None else
+        pyvars(object)
+    )
 
-    return builtins.vars(object)
+pydir = builtins.dir
 
 @PysPythonFunction
 def dir(pyfunc, *args):
-    if len(args) == 0:
-        return list(pyfunc.__code__.context.symbol_table.symbols.keys())
-
-    return builtins.dir(*args)
+    return (
+        pydir(*args)
+        if args else
+        list(pyfunc.__code__.context.symbol_table.symbols.keys())
+    )
 
 @PysPythonFunction
 def exec(pyfunc, source, globals=None):
@@ -207,21 +208,25 @@ def exec(pyfunc, source, globals=None):
         raise TypeError("exec(): globals must be dict")
 
     file = PysFileBuffer(source, '<exec>')
+    code = pyfunc.__code__
+
+    if globals is None:
+        symtab = code.context.symbol_table
+    else:
+        symtab, _ = new_symbol_table(symbols=globals)
 
     from .runner import pys_runner
 
     result = pys_runner(
         file=file,
         mode='exec',
-        symbol_table=pyfunc.__code__.context.symbol_table
-                     if globals is None else
-                     build_symbol_table(file, globals),
-        context_parent=pyfunc.__code__.context,
-        context_parent_entry_position=pyfunc.__code__.position
+        symbol_table=symtab,
+        context_parent=code.context,
+        context_parent_entry_position=code.position
     )
 
     if result.error:
-        raise PysShouldReturn(PysRunTimeResult().failure(result.error))
+        raise PysSignal(PysRunTimeResult().failure(result.error))
 
 @PysPythonFunction
 def eval(pyfunc, source, globals=None):
@@ -229,21 +234,25 @@ def eval(pyfunc, source, globals=None):
         raise TypeError("eval(): globals must be dict")
 
     file = PysFileBuffer(source, '<eval>')
+    code = pyfunc.__code__
+
+    if globals is None:
+        symtab = code.context.symbol_table
+    else:
+        symtab, _ = new_symbol_table(symbols=globals)
 
     from .runner import pys_runner
 
     result = pys_runner(
         file=file,
         mode='eval',
-        symbol_table=pyfunc.__code__.context.symbol_table
-                     if globals is None else
-                     build_symbol_table(file, globals),
-        context_parent=pyfunc.__code__.context,
-        context_parent_entry_position=pyfunc.__code__.position
+        symbol_table=symtab,
+        context_parent=code.context,
+        context_parent_entry_position=code.position
     )
 
     if result.error:
-        raise PysShouldReturn(PysRunTimeResult().failure(result.error))
+        raise PysSignal(PysRunTimeResult().failure(result.error))
 
     return result.value
 
@@ -313,9 +322,8 @@ def comprehension(init, wrap, condition=None):
 
     return map(wrap, init if condition is None else filter(condition, init))
 
-pys_builtins = PysModule(
+pys_builtins = ModuleType(
     'built-in',
-
     "Built-in functions, types, exceptions, and other objects.\n\n"
     "This module provides direct access to all 'built-in' identifiers of PyScript and Python."
 )
@@ -323,10 +331,13 @@ pys_builtins = PysModule(
 pys_builtins.__dict__.update(
     (name, getattr(builtins, name))
     for name in builtins.dir(builtins)
-    if not (name.startswith('_') or name in BLACKLIST_PYTHON_BUILTINS)
+    if not (is_private_attribute(name) or is_blacklist_python_builtins(name))
 )
 
 pys_builtins.__file__ = __file__
+pys_builtins.true = True
+pys_builtins.false = False
+pys_builtins.none = None
 pys_builtins.license = license
 pys_builtins.help = help
 pys_builtins.pyimport = pyimport
