@@ -1,21 +1,23 @@
 from .analyzer import PysAnalyzer
 from .buffer import PysFileBuffer
 from .cache import undefined, hook, PysUndefined
-from .constants import DEFAULT, SILENT, RETRES, COMMENT, NO_COLOR, BOLD
+from .constants import PYSCRIPT_SHELL, PYSCRIPT_TYPECHECKING, DEFAULT, SILENT, RETRES, COMMENT, NO_COLOR
 from .context import PysContext
-from .exceptions import PysException
+from .exceptions import PysException, PysSignal
 from .handlers import handle_exception, handle_execute
 from .interpreter import visit
 from .lexer import PysLexer
 from .parser import PysParser
 from .position import PysPosition
 from .results import PysRunTimeResult, PysExecuteResult
-from .symtab import PysSymbolTable, build_symbol_table
+from .symtab import PysSymbolTable, new_symbol_table
+from .utils.ansi import BOLD, acolor
 from .utils.debug import print_display
-from .utils.decorators import typechecked, _TYPECHECKED
-from .utils.general import setimuattr, acolor, get_locals, is_object_of
+from .utils.decorators import typechecked
+from .utils.generic import setimuattr, get_locals
 from .version import version
 
+from os import environ
 from sys import stderr, version as pyversion
 from typing import Any, Literal, Optional
 
@@ -23,14 +25,21 @@ def _normalize_globals(file, globals, stack_level):
     stack_level += 1
 
     if globals is None:
-        globals = build_symbol_table(file, get_locals(stack_level + 1 if _TYPECHECKED else stack_level))
-    elif globals is undefined:
-        globals = build_symbol_table(file)
-        globals.set('__name__', '__main__')
-    elif isinstance(globals, dict):
-        globals = build_symbol_table(file, globals)
+        symtab, _ = new_symbol_table(
+            symbols=get_locals(
+                stack_level + 1
+                if environ.get(PYSCRIPT_TYPECHECKING, '1') == '1' else
+                stack_level
+            )
+        )
 
-    return globals
+    elif globals is undefined:
+        symtab, _ = new_symbol_table(file=file.name, name='__main__')
+
+    elif isinstance(globals, dict):
+        symtab, _ = new_symbol_table(symbols=globals)
+
+    return symtab
 
 @typechecked
 def pys_runner(
@@ -102,7 +111,6 @@ def pys_runner(
             )
 
         setimuattr(context, 'flags', parser.flags)
-
         runtime_result = visit(ast.node, context)
 
         return (
@@ -148,7 +156,7 @@ def pys_exec(
         return result
 
     elif result.error and not (flags & SILENT):
-        raise result.error.exception
+        raise PysSignal(PysRunTimeResult().failure(result.error))
 
 @typechecked
 def pys_eval(
@@ -184,7 +192,7 @@ def pys_eval(
         return result
 
     elif result.error and not (flags & SILENT):
-        raise result.error.exception
+        raise PysSignal(PysRunTimeResult().failure(result.error))
 
     return result.value
 
@@ -206,12 +214,12 @@ def pys_shell(
     flags: A special flags.
     """
 
+    if environ.get(PYSCRIPT_SHELL, '0') == '1':
+        raise RuntimeError("another shell is still running")
+
     file = PysFileBuffer('', '<pyscript-shell>')
-
     globals = _normalize_globals(file, globals, 2)
 
-    hook.display = print_display
-
     if flags & NO_COLOR:
         reset = ''
         bmagenta = ''
@@ -243,110 +251,120 @@ def pys_shell(
 
     print(f'PyScript {version}')
     print(f'Python {pyversion}')
-    print('Type "help" or "license" for more information.')
+    print('Type "help" or "license" for more information; "exit" or "/exit" to exit the shell.')
 
-    while True:
+    try:
 
-        try:
+        environ[PYSCRIPT_SHELL] = '1'
+        hook.display = print_display
 
-            if is_next_line():
-                text = input(f'{bmagenta}{hook.ps2}{reset}')
+        while True:
 
-            else:
-                text = input(f'{bmagenta}{hook.ps1}{reset}')
-                if text == '/exit':
-                    return 0
+            try:
 
-            next_line = False
-            in_decorator = False
-            is_space = True
+                if is_next_line():
+                    text = input(f'{bmagenta}{hook.ps2}{reset}')
+                else:
+                    text = input(f'{bmagenta}{hook.ps1}{reset}')
+                    if text == '/exit':
+                        return 0
 
-            i = 0
-
-            while i < len(text):
-                character = text[i]
-
-                if character == '\\':
-                    i += 1
-                    character = text[i:i+1]
-
-                    if character == '':
-                        next_line = True
-                        break
-
-                elif character in '\'"':
-                    bind_3 = text[i:i+3]
-
-                    if is_triple_string:
-
-                        if len(bind_3) == 3 and string_prefix * 3 == bind_3:
-                            in_string = False
-                            is_triple_string = False
-                            i += 2
-
-                    else:
-                        if not in_string and bind_3 in ("'''", '"""'):
-                            is_triple_string = True
-                            i += 2
-
-                        if in_string and string_prefix == character:
-                            in_string = False
-                        else:
-                            string_prefix = character
-                            in_string = True
-
-                if not in_string:
-
-                    if character == '#':
-                        break
-
-                    elif is_space and character == '@':
-                        in_decorator = True
-                        i += 1
-                        continue
-
-                    elif character in '([{':
-                        parenthesis_level += 1
-
-                    elif character in ')]}':
-                        parenthesis_level -= 1
-
-                    if not character.isspace():
-                        is_space = False
-
-                i += 1
-
-            if in_decorator and is_space:
+                next_line = False
                 in_decorator = False
+                is_space = True
+                i = 0
 
-            if in_string and not (next_line or is_triple_string):
-                in_string = False
-                parenthesis_level = 0
+                while i < len(text):
+                    character = text[i]
 
-            if is_next_line():
-                full_text += text + '\n'
-                continue
+                    if character == '\\':
+                        i += 1
+                        character = text[i:i+1]
 
-            result = pys_runner(
-                file=PysFileBuffer(full_text + text, f'<pyscript-shell-{line}>'),
-                mode='exec',
-                symbol_table=globals,
-                flags=flags
-            )
+                        if character == '':
+                            next_line = True
+                            break
 
-            if result.error and is_object_of(result.error.exception, SystemExit):
-                return result.error.exception.code
+                    elif character in '\'"':
+                        bind_3 = text[i:i+3]
 
-            flags = result.context.flags
-            code = handle_execute(result)
-            if code == 0:
-                line += 1
+                        if is_triple_string:
+                            if len(bind_3) == 3 and string_prefix * 3 == bind_3:
+                                in_string = False
+                                is_triple_string = False
+                                i += 2
 
-            reset_next_line()
+                        else:
+                            if not in_string and bind_3 in ("'''", '"""'):
+                                is_triple_string = True
+                                i += 2
 
-        except KeyboardInterrupt:
-            reset_next_line()
-            print(f'\r{bmagenta}KeyboardInterrupt. Type "exit" or "/exit" to exit.{reset}', file=stderr)
+                            if in_string and string_prefix == character:
+                                in_string = False
+                            else:
+                                string_prefix = character
+                                in_string = True
 
-        except EOFError:
-            return 0
\ No newline at end of file
+                    if not in_string:
+
+                        if character == '#':
+                            break
+
+                        elif is_space and character == '@':
+                            in_decorator = True
+                            i += 1
+                            continue
+
+                        elif character in '([{':
+                            parenthesis_level += 1
+
+                        elif character in ')]}':
+                            parenthesis_level -= 1
+
+                        if not character.isspace():
+                            is_space = False
+
+                    i += 1
+
+                if in_decorator and is_space:
+                    in_decorator = False
+
+                if in_string and not (next_line or is_triple_string):
+                    in_string = False
+                    parenthesis_level = 0
+
+                if is_next_line():
+                    full_text += text + '\n'
+                    continue
+
+                result = pys_runner(
+                    file=PysFileBuffer(full_text + text, f'<pyscript-shell-{line}>'),
+                    mode='exec',
+                    symbol_table=globals,
+                    flags=flags
+                )
+
+                if result.error:
+                    if result.error.exception is SystemExit:
+                        return 0
+                    if type(result.error.exception) is SystemExit:
+                        return result.error.exception.code
+
+                flags = result.context.flags
+                code = handle_execute(result)
+                if code == 0:
+                    line += 1
+
+                reset_next_line()
+
+            except KeyboardInterrupt:
+                reset_next_line()
+                print(f'\r{bmagenta}KeyboardInterrupt{reset}', file=stderr)
+
+            except EOFError:
+                print()
+                return 0
+
+    finally:
+        environ[PYSCRIPT_SHELL] = '0'
+        hook.display = None
\ No newline at end of file
